/**
 *
 */
package org.n52.sos.decode.json.inspire;

import org.n52.sos.inspire.aqd.Address;
import org.n52.sos.inspire.aqd.GeographicalName;
import org.n52.sos.ogc.ows.OwsExceptionReport;
import org.n52.sos.util.AQDJSONConstants;

import com.fasterxml.jackson.databind.JsonNode;

public class AddressJSONDecoder extends AbstractJSONDecoder<Address> {

    public AddressJSONDecoder() {
        super(Address.class);
    }

    @Override
    public Address decodeJSON(JsonNode node, boolean validate)
            throws OwsExceptionReport {
        Address address = new Address();
        address.setAddressFeature(parseNillableReference(node
                .path(AQDJSONConstants.ADDRESS_FEATURE)));
        address.setPostCode(parseNillableString(node
                .path(AQDJSONConstants.POST_CODE)));
        for (JsonNode n : node.path(AQDJSONConstants.ADDRESS_AREAS)) {
            address.addAddressArea(decodeJsonToNillable(n, GeographicalName.class));
        }
        for (JsonNode n : node.path(AQDJSONConstants.ADMIN_UNITS)) {
            address.addAdminUnit(decodeJsonToObject(n, GeographicalName.class));
        }
        for (JsonNode n : node.path(AQDJSONConstants.LOCATOR_DESIGNATORS)) {
            address.addLocatorDesignator(n.textValue());
        }
        for (JsonNode n : node.path(AQDJSONConstants.LOCATOR_NAMES)) {
            address.addLocatorName(decodeJsonToObject(n, GeographicalName.class));
        }
        for (JsonNode n : node.path(AQDJSONConstants.POST_NAMES)) {
            address.addPostName(decodeJsonToNillable(n, GeographicalName.class));
        }
        for (JsonNode n : node.path(AQDJSONConstants.THOROUGHFARES)) {
            address.addThoroughfare(decodeJsonToNillable(n, GeographicalName.class));
        }
        return address;
    }
}
