/**
 *
 */
package org.n52.sos.decode.xml.stream;

import java.util.Arrays;
import java.util.List;

import javax.xml.namespace.QName;
import javax.xml.stream.XMLStreamException;

import org.n52.sos.aqd.AqdConstants;
import org.n52.sos.iso.GcoConstants;
import org.n52.sos.ogc.ows.OwsExceptionReport;
import org.n52.sos.util.Nillable;
import org.n52.sos.w3c.W3CConstants;

import com.google.common.base.Optional;
import com.google.common.collect.Iterables;

/**
 * TODO JavaDoc
 *
 * @author Christian Autermann
 */
public abstract class NillableReader<T> extends XmlReader<Nillable<T>> {
    private Nillable<T> nillable;

    protected abstract XmlReader<T> getDelegate();

    protected List<QName> getPossibleNilReasonAttributes() {
        return Arrays.asList(AqdConstants.QN_NIL_REASON,
                             GcoConstants.QN_GCO_NIL_REASON);
    }

    @Override
    protected void begin()
            throws XMLStreamException, OwsExceptionReport {
        Optional<String> attr = attr(W3CConstants.QN_XSI_NIL);
        if (attr.isPresent() && attr.get().equals("true")) {
            List<QName> attributeNames = getPossibleNilReasonAttributes();
            Iterable<Optional<String>> attributes = attr(attributeNames);
            Iterable<String> reasons = Optional.presentInstances(attributes);
            this.nillable = Nillable.nil(Iterables.getFirst(reasons, null));
        } else {
            this.nillable = Nillable.of(delegate(getDelegate()));
        }
    }

    @Override
    protected Nillable<T> finish() {
        return nillable;
    }

}
